searchData={"items":[{"type":"module","title":"ServerSentEvents","doc":"This module implements a parser for Server Sent Events. See the official\nspecification for details on parsing and interpreting the event stream.\n\n- https://html.spec.whatwg.org/multipage/server-sent-events.html","ref":"ServerSentEvents.html"},{"type":"function","title":"ServerSentEvents.parse/1","doc":"Parses a chunk of data into a list of Server Sent Event messages.\n\nReturns a tuple containing the list of parsed events and the remaining data\nfrom the chunk if it contained an incomplete event.\n\nThere are only four possible keys in the event map: `:id`, `:event`, `:data`, `:retry`.","ref":"ServerSentEvents.html#parse/1"},{"type":"function","title":"Examples - ServerSentEvents.parse/1","doc":"iex> ServerSentEvents.parse(\"event: event\\ndata: {\\\"complete\\\":\")\n    {[], \"event: event\\ndata: {\\\"complete\\\":\"}\n\n    iex> ServerSentEvents.parse(\"event: event\\ndata: {\\\"complete\\\":true}\\n\\n\")\n    {[%{event: \"event\", data: \"{\\\"complete\\\":true}\"}], \"\"}","ref":"ServerSentEvents.html#parse/1-examples"},{"type":"extras","title":"ServerSentEvents","doc":"# ServerSentEvents\n\n[![CI](https://github.com/benjreinhart/server_sent_events/actions/workflows/ci.yml/badge.svg)](https://github.com/benjreinhart/server_sent_events/actions/workflows/ci.yml)\n[![License](https://img.shields.io/hexpm/l/server_sent_events.svg)](https://github.com/benjreinhart/server_sent_events/blob/main/LICENSE.md)\n[![Version](https://img.shields.io/hexpm/v/server_sent_events.svg)](https://hex.pm/benjreinhart/server_sent_events)\n\nLightweight, ultra-fast Server Sent Event parser for Elixir.\n\nThis module fully conforms to the official [Server Sent Events specification](https://html.spec.whatwg.org/multipage/server-sent-events.html#parsing-an-event-stream) with a comprehensive [test suite](https://github.com/benjreinhart/server_sent_events/blob/main/test/server_sent_events_test.exs).","ref":"readme.html"},{"type":"extras","title":"Usage - ServerSentEvents","doc":"```elixir\n{events, rest} = ServerSentEvents.parse(\"event: event\\ndata: {\\\"complete\\\":true}\\n\\n\")\nIO.inspect(events)   # [%{event: \"event\", data: \"{\\\"complete\\\":true}\\n\"}]\nIO.inspect(rest)     # \"\"\n```\n\nParsing a chunk containing zero or more events followed by an incomplete event returns the incomplete data.\n\n```elixir\n{events, buffer} = ServerSentEvents.parse(\"event: event\\ndata: {\\\"complete\\\":\")\nIO.inspect(events)   # []\nIO.inspect(buffer)   # \"event: event\\ndata: {\\\"complete\\\":\"\n\n{events, buffer} = ServerSentEvents.parse(buffer <> \"true}\\n\\nevent: event\\ndata: {\")\nIO.inspect(events)   # [%{event: \"event\", data: \"{\\\"complete\\\":true}\\n\"}]\nIO.inspect(buffer)   # \"event: event\\ndata: {\"\n\n{events, rest} = ServerSentEvents.parse(buffer <> \"\\\"key\\\":\\\"value\\\"}\\n\\n\")\nIO.inspect(events)   # [%{event: \"event\", data: \"{\\\"key\\\":\\\"value\\\"}\\n\"}]\nIO.inspect(rest)     # \"\"\n```\n\nThis can be useful for streaming environments where a single event may not reliably arrive in one chunk.","ref":"readme.html#usage"},{"type":"extras","title":"Real world example - ServerSentEvents","doc":"AI providers like OpenAI and Anthropic stream AI generated messages using Server Sent Events.\nThis module can handle parsing the server sent events, returning a list of maps. For example,\nwe can parse a streaming response from Anthropic:\n\n```elixir\nReq.post(\"https://api.anthropic.com/v1/messages\",\n  json: request,\n  into: fn {:data, data}, {req, res} ->\n    buffer = Request.get_private(req, :sse_buffer, \"\")\n    {events, buffer} = ServerSentEvents.parse(buffer <> data)\n    Request.put_private(req, :sse_buffer, buffer)\n\n    if events != [] do\n      # Do something with events, e.g., send to a process consuming them.\n      send(pid, {:events, events})\n    end\n\n    {:cont, {req, res}}\n  end,\n  headers: %{\n    \"x-api-key\" => api_key(),\n    \"anthropic-version\" => \"2023-06-01\"\n  }\n)\n```\n\nThe first chunk from Anthropic tends to contain a couple of messages that look something like the following when parsed by this module:\n\n```elixir\n# Parsing first chunk from Anthropic\n{events, \"\"} = ServerSentEvents.parse(chunk)\nIO.inspect(events)\n# [\n#   %{\n#     data: \"{\\\"type\\\":\\\"message_start\\\",\\\"message\\\":{\\\"id\\\":\\\"msg_01LAFhYgKvtBB5ac5n41oyDn\\\",\\\"type\\\":\\\"message\\\",\\\"role\\\":\\\"assistant\\\",\\\"model\\\":\\\"claude-3-5-sonnet-20241022\\\",\\\"content\\\":[],\\\"stop_reason\\\":null,\\\"stop_sequence\\\":null,\\\"usage\\\":{\\\"input_tokens\\\":12,\\\"output_tokens\\\":2}}        }\",\n#     event: \"message_start\"\n#   },\n#   %{\n#     data: \"{\\\"type\\\":\\\"content_block_start\\\",\\\"index\\\":0,\\\"content_block\\\":{\\\"type\\\":\\\"text\\\",\\\"text\\\":\\\"\\\"}         }\",\n#     event: \"content_block_start\"\n#   },\n#   %{data: \"{\\\"type\\\": \\\"ping\\\"}\", event: \"ping\"},\n#   %{\n#     data: \"{\\\"type\\\":\\\"content_block_delta\\\",\\\"index\\\":0,\\\"delta\\\":{\\\"type\\\":\\\"text_delta\\\",\\\"text\\\":\\\"Here's\\\"}          }\",\n#     event: \"content_block_delta\"\n#   }\n# ]\n```\n\nThis is typically followed by filtering out unwanted events and JSON parsing the `data` field of meaningful events.","ref":"readme.html#real-world-example"},{"type":"extras","title":"Installation - ServerSentEvents","doc":"The package can be installed by adding `server_sent_events` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:server_sent_events, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"v0.2.1 (2024-11-11) - CHANGELOG","doc":"- Add GitHub CI\n- Add readme for use in hex docs","ref":"changelog.html#v0-2-1-2024-11-11"},{"type":"extras","title":"v0.2.0 (2024-11-11) - CHANGELOG","doc":"- Use symbols for map keys in `parse/1`'s return value\n- Small refactorings\n- Update docs","ref":"changelog.html#v0-2-0-2024-11-11"},{"type":"extras","title":"v0.1.0 (2024-11-10) - CHANGELOG","doc":"- Initial publish","ref":"changelog.html#v0-1-0-2024-11-10"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}